// inspect.cpp

// Copyright (c) 2006 Johan Rade

// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt
// or copy at http://www.boost.org/LICENSE_1_0.txt)

//-------------------------------------

#include <iomanip>
#include <iostream>
#include <limits>

#include <boost/detail/endian.hpp>

//------------------------------------------------------------------------------

template<class T> void print(const char* s, T x, bool ok = true);
template<class T> void print_info();

//------------------------------------------------------------------------------

int main()
{
	std::cout << '\n';

#if defined BOOST_BIG_ENDIAN
    std::cout << "big endian\n\n";
#elif defined BOOST_LITTLE_ENDIAN
	std::cout << "little endian\n\n";
#elif defined BOOST_PDP_ENDIAN
#	error PDP-endian platforms are not supported
#else
#	error should not get here
#endif

	std::cout << "float -------------------------------\n\n";
	print_info<float>();
	std::cout << "double ------------------------------\n\n";
	print_info<double>();
	std::cout << "long double -------------------------\n\n";
	print_info<long double>();

    return 0;
}

//------------------------------------------------------------------------------

template<class T>
void print(const char* s, T x, bool ok)
{

	std::cout << std::left << std::setw(10) << s << std::right;

	std::cout << std::hex << std::setfill('0');

	if(ok) {
#ifdef BOOST_BIG_ENDIAN
		for(int i = 0; i < (int)sizeof(T); ++i)
#else
		for(int i = sizeof(T) - 1; i >= 0; --i)
#endif
			std::cout << std::setw(2) << (unsigned int)*((unsigned char*)&x + i) << ' ';
	}
	else {
		for(int i = 0; i < (int)sizeof(T); ++i)
			std::cout << "-- ";
	}

	std::cout << '\n';
	std::cout << std::dec << std::setfill(' ');
}

template<class T> void print_info()
{
	print("0", (T)0);
	print("sn. min", std::numeric_limits<T>::denorm_min());
	print("n. min", std::numeric_limits<T>::min());
	print("1", (T)1);
	print("4/3", (T)4/(T)3);
	print("max", std::numeric_limits<T>::max());
	print("inf", std::numeric_limits<T>::infinity(), std::numeric_limits<T>::has_infinity);
	print("q. nan", std::numeric_limits<T>::quiet_NaN(), std::numeric_limits<T>::has_quiet_NaN);
	print("s. nan", std::numeric_limits<T>::signaling_NaN(), std::numeric_limits<T>::has_signaling_NaN);
	print("-1", (T)-1);

	std::cout << "\n\n";
}

